/*
 * xstat - A statistics generator using GTK+
 *
 * Copyright (C) 2000 Julien Lesouef <julien.lesouef@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */


#include "User.h"


/*
 * Determines one chance in ten.
 */
bool User::hasChance()
{
  bool ret;
  int i;
  i = (int) (10.0 * rand() / (RAND_MAX+1.0) );
  if (i == 0) ret = 1;
  else ret = 0;
  return ret;
}


/*
 * Copies fromSentence in toSentence with the limit of 'SENSIZE' characters.
 */
void User::copy(char toSentence[], char fromSentence[])
{
  strncpy(toSentence, fromSentence, SENSIZE);
  if (strlen(fromSentence) > SENSIZE - 1)
    {
      toSentence[SENSIZE - 4] = '.';
      toSentence[SENSIZE - 3] = '.';
      toSentence[SENSIZE - 2] = '.';
      toSentence[SENSIZE - 1] = '\0';
    }
}


/*
 * Constructs an user with an user name and a domain name.
 */
User::User(char aName[], char aDomain[])
{
  strcpy(name, aName);
  strcpy(domain, aDomain);
  nbSession = 0;
  strcpy(timeSentence, "No sentence\0");
  strcpy(quitSentence, "No sentence\0");
  nbLine = 0;
  strcpy(lineSentence, "No sentence\0");
  nbTopic = 0;
  strcpy(topicSentence, "No sentence\0");
  nbAction = 0;
  strcpy(actionSentence, "No sentence\0");
  nbKick = 0;
  strcpy(kickSentence, "No sentence\0");
  nbKicked = 0;
  strcpy(kickedSentence, "No sentence\0");
}


/*
 * Destroys the user.
 */
User::~User()
{
  for (int i = 0; i <= nbSession; i++)
    {
      delete session[i];
    }
}


/*
 * Indicates that the user has joined the channel
 * with the specified nick and at he specified time.
 */
void User::joins(char aNick[], time_t aTime)
{
  session[nbSession] = new Session(aNick, aTime);
}


/*
 * Increases the number of lines for the user.
 * May take the sentence in parameter for lineSentence or timeSentence.
 */
void User::speaks(char aSentence[])
{
  nbLine++;
  if (nbLine == 1) 
    {
      copy(lineSentence, aSentence);
      copy(timeSentence, aSentence);
    }
  if (hasChance())
    {
      copy(lineSentence, aSentence);
    }
  if (hasChance())
    {
      copy(timeSentence, aSentence);
    }
}


/*
 * Increases the number of actions for the user.
 * May take the sentence in parameter for actionSentence.
 */
void User::doesAction(char aSentence[])
{
  nbAction++;
  if (nbAction == 1) 
    {
      copy(actionSentence, aSentence);
    }
  if (hasChance())
    {
      copy(actionSentence, aSentence);
    }
}


/*
 * Increases the number of topics for the user.
 * May take the sentence in parameter for topicSentence.
 */
void User::changesTopic(char aSentence[])
{
  nbTopic++;
  if (nbTopic == 1) 
    {
      copy(topicSentence, aSentence);
    }
  if (hasChance())
    {
      copy(topicSentence, aSentence);
    }
}


/* 
 * Increases the number of kick made by the user.
 * May take the sentence in parameter for kickSentence.
 */
void User::kicks(char aSentence[])
{
  nbKick++;
  if (nbKick == 1) 
    {
      copy(kickSentence, aSentence);
    }
  if (hasChance())
    {
      copy(kickSentence, aSentence);
    }
}


/* 
 * Indicates that the user has been kicked
 * at the specified time.
 */
void User::isKicked(char aSentence[], time_t aTime)
{  
  nbKicked++;
  if (nbKicked == 1) 
    {
      copy(kickedSentence, aSentence);
    }
  if (hasChance())
    {
      copy(kickedSentence, aSentence);
    }
  session[nbSession]->leaves(aTime);
  nbSession++;
  session[nbSession] = NULL;
}


/* 
 * Indicates that the user has left the channel
 * at the specified time.
 */
void User::leaves(time_t aTime)
{
  session[nbSession]->leaves(aTime);
  nbSession++;
  session[nbSession] = NULL;
}


/* 
 * Indicates that the user has left IRC
 * at the specified time.
 */
void User::leavesIRC(char aSentence[], time_t aTime)
{
  if (!strcmp(quitSentence, "No sentence\0")) 
    {
      copy(quitSentence, aSentence);
    }
  if (hasChance())
    {
      copy(quitSentence, aSentence);
    }
  session[nbSession]->leaves(aTime);
  nbSession++;
  session[nbSession] = NULL;
}


/*
 * Determines if the specified nick is the same as
 * the nick of the current session.
 */
bool User::compareNick(char aNick[])
{
  bool ret = 0;
  if (session[nbSession] != NULL)
    {
      ret = (!strcmp(session[nbSession]->getNick(), aNick));
    }
  return ret;
}


/*
 * Determnes if the specified address is the same as
 * the address of the user.
 */
bool User::compareAddress(char aName[], char aDomain[])
{
  return (((strstr(name, aName) != NULL) || (strstr(aName, name) != NULL))
	  && (!strcmp(domain, aDomain)));
}


/*
 * Gets the number of sessions of the user.
 */
int User::getNbSessions()
{
  return nbSession;
}


/*
 * Gets the sentence of "quit" of the last session of the user.
 */
char *User::getLeaveSentence()
{
  return quitSentence;
}


/*
 * Determines if the user was present at the specified time.
 */
bool User::wasPresent(time_t aTime)
{
  bool ret = 0;
  int i;
  for (i = 0; i < nbSession; i++)
    {
      if (session[i]->wasPresent(aTime)) ret = 1;
    }
  return ret;
}


/*
 * Gets the number of different nicks of the user.
 */
int User::getNbNicks()
{
  int i, j, ret;
  bool tmp;

  ret = 0;
  for (i = 0; i < nbSession; i++)
    {
      tmp = 1;
      for (j = 0; j < i; j++)
	{
	  if (!strcmp(session[i]->getNick(), session[j]->getNick())) tmp = 0;
	}
      if (tmp == 1) ret++;
    }
  return ret;
}


/**
 * Gets the most used nick of the user.
 */
char *User::getMostUsedNick()
{
  int max, tmp, i, j ,indice;
  
  max = 0;
  for (i = 0; i < nbSession; i++)
    {
      tmp = 0;
      for (j = 0; j < nbSession; j++)
	{
	  if (!strcmp(session[i]->getNick(), session[j]->getNick())) tmp++;
	}
      if (tmp > max)
	{
	  max = tmp;
	  indice = i;
	}
    }
  return session[indice]->getNick();
}


/*
 * Gets all nicks of the user separated by commas. 
 */
char *User::getAllNicks()
{
  char *ret = new char[500];
  int i, j;
  bool tmp;

  ret = strcpy(ret, "\0");
  for (i = 0; i < nbSession; i++)
    {
      tmp = 1;
      for (j = 0; j < i; j++)
	{
	  if (!strcmp(session[i]->getNick(), session[j]->getNick())) tmp = 0;
	}
      if (tmp) 
	{
	  if (i > 0) ret = strcat(ret, ", ");
	  ret = strcat(ret, session[i]->getNick());
	}
    }
  ret = strcat(ret, "\0");
  return ret;
}


/*
 * Gets the duration of the user.
 */
time_t User::getDuration()
{
  time_t ret = 0;
  int i;
  for (i = 0; i < nbSession; i++)
    {
      if (i == 0) ret = session[i]->getDuration();
      ret += session[i]->getDuration();
    }
  return ret;
}


/*
 * Gets a hazard sentence of the user.
 */
char *User::getDurationSentence()
{
  return timeSentence;
}


/*
 * Gets the number of lines of the user.
 */
int User::getNbLines()
{
  return nbLine;
}


/*
 * Gets a hazard sentence of the user.
 */
char *User::getLineSentence()
{
  return lineSentence;
}


/*
 * Gets the number of actions of the user.
 */
int User::getNbActions()
{
  return nbAction;
}


/*
 * Gets a hazard action of the user.
 */
char *User::getActionSentence()
{
  return actionSentence;
}


/*
 * Gets the number of topics changed by the user.
 */
int User::getNbTopics()
{
  return nbTopic;
}


/*
 * Gets a hazard topic changed by the user.
 */
char *User::getTopicSentence()
{
  return topicSentence;
}


/*
 * Gets the number of kicks done by the user.
 */
int User::getNbKicks()
{
  return nbKick;
}


/*
 * Gets a hazard kick sentence of the user.
 */
char *User::getKickSentence()
{
  return kickSentence;
}


/*
 * Gets the number of kicks done on the user.
 */
int User::getNbKickeds()
{
  return nbKicked;
}


/*
 * Gets a hazard kick sentence of the user kicked.
 */
char *User::getKickedSentence()
{
  return kickedSentence;
}
